@@ -386,7 +386,7 @@ void libvisio::VSDXParser::handleStencilForeign(WPXInputStream *input, unsigned
     m_input->seek(ptrOffset, WPX_SEEK_SET);
     VSDInternalStream tmpInput(m_input, ptrLength, compressed);
 
-    VSD_DEBUG_MSG(("Stencil foreign stream %x\n", ptrType));
+    shift = compressed ? 4 : 0;
 
     if (ptrType == VSD_PROP_LIST)
     {
@@ -433,6 +433,56 @@ void libvisio::VSDXParser::handleStencilForeign(WPXInputStream *input, unsigned
         m_stencilShape.m_foreign->data = binaryData;
       }
     }
+    else if (ptrType == VSD_OLE_LIST)
+    {
+      m_stencilShape.m_foreign->dataId = m_header.id;
+      handleStencilOle(&tmpInput, shift);
+    }
+  }
+}
+
+void libvisio::VSDXParser::handleStencilOle(WPXInputStream *input, unsigned shift)
+{
+  unsigned ptrType;
+  unsigned ptrOffset;
+  unsigned ptrLength;
+  unsigned ptrFormat;
+
+  input->seek(shift, WPX_SEEK_CUR);
+  unsigned offset = readU32(input);
+  input->seek(offset+shift, WPX_SEEK_SET);
+  unsigned pointerCount = readU32(input);
+  input->seek(4, WPX_SEEK_CUR); // Ignore 0x0 dword
+
+  for (unsigned i = 0; i < pointerCount; i++)
+  {
+    ptrType = readU32(input);
+    input->seek(4, WPX_SEEK_CUR); // Skip dword
+    ptrOffset = readU32(input);
+    ptrLength = readU32(input);
+    ptrFormat = readU16(input);
+
+    bool compressed = ((ptrFormat & 2) == 2);
+    m_input->seek(ptrOffset, WPX_SEEK_SET);
+    VSDInternalStream tmpInput(m_input, ptrLength, compressed);
+
+    shift = compressed ? 4 : 0;
+    tmpInput.seek(shift, WPX_SEEK_CUR);
+
+    if (ptrType == VSD_OLE_DATA)
+    {
+      // Be sure to use internal stream size to get decompressed size
+      unsigned foreignLength = tmpInput.getSize() - shift;
+      unsigned long tmpBytesRead = 0;
+      const unsigned char *buffer = tmpInput.read(foreignLength, tmpBytesRead);
+
+      if (foreignLength == tmpBytesRead)
+      {
+        // Append data instead of setting it - allows multi-stream OLE objects
+        m_stencilShape.m_foreign->data.append(buffer, tmpBytesRead);
+        m_stencilShape.m_foreign->dataLevel = m_header.level;
+      }
+    }
   }
 }
 
@@ -579,21 +629,21 @@ void libvisio::VSDXParser::_handleLevelChange(unsigned level)
   }
   if (level < 2)
   {
-    for (std::vector<VSDXGeometryList *>::iterator iter = m_geomListVector.begin(); iter != m_geomListVector.end(); iter++)
+    for (std::vector<VSDXGeometryList *>::iterator iter = m_geomListVector.begin(); iter != m_geomListVector.end(); ++iter)
     {
       (*iter)->handle(m_collector);
       (*iter)->clear();
       delete *iter;
     }
     m_geomListVector.clear();
-    for (std::vector<VSDXCharacterList *>::iterator iter2 = m_charListVector.begin(); iter2 != m_charListVector.end(); iter2++)
+    for (std::vector<VSDXCharacterList *>::iterator iter2 = m_charListVector.begin(); iter2 != m_charListVector.end(); ++iter2)
     {
       (*iter2)->handle(m_collector);
       (*iter2)->clear();
       delete *iter2;
     }
     m_charListVector.clear();
-    for (std::vector<VSDXParagraphList *>::iterator iter3 = m_paraListVector.begin(); iter3 != m_paraListVector.end(); iter3++)
+    for (std::vector<VSDXParagraphList *>::iterator iter3 = m_paraListVector.begin(); iter3 != m_paraListVector.end(); ++iter3)
     {
       (*iter3)->handle(m_collector);
       (*iter3)->clear();
@@ -1182,7 +1232,6 @@ void libvisio::VSDXParser::readNURBSTo(WPXInputStream *input)
   unsigned cellRef = 0;
   unsigned length = 0;
   unsigned long inputPos = input->tell();
-  unsigned long bytesRead = 0;
   while (cellRef != 6 && !input->atEOS() &&
          m_header.dataLength - chunkBytesRead > 4)
   {
@@ -1198,12 +1247,12 @@ void libvisio::VSDXParser::readNURBSTo(WPXInputStream *input)
   if (input->atEOS())
     return;
 
-  unsigned degree = 3;
-  unsigned char xType = 1;
-  unsigned char yType = 1;
   // Only read formula if block is found
   if (cellRef == 6)
   {
+    unsigned char xType = 1;
+    unsigned char yType = 1;
+    unsigned degree = 3;
     // Indicates whether it's a "simple" NURBS block with a static format
     // or a complex block where parameters each have a type
     unsigned char paramType = readU8(input);
@@ -1238,7 +1287,7 @@ void libvisio::VSDXParser::readNURBSTo(WPXInputStream *input)
     }
 
     // Read sequences of (x, y, knot, weight) until finished
-    bytesRead = input->tell() - inputPos;
+    unsigned long bytesRead = input->tell() - inputPos;
     unsigned char flag = 0;
     if (paramType != 0x8a) flag = readU8(input);
     while ((flag != 0x81 || (paramType == 0x8a && repetitions > 0)) && bytesRead < length)
@@ -1339,7 +1388,6 @@ void libvisio::VSDXParser::readPolylineTo(WPXInputStream *input)
   unsigned cellRef = 0;
   unsigned length = 0;
   unsigned long inputPos = input->tell();
-  unsigned long blockBytesRead = 0;
   while (cellRef != 2 && !input->atEOS() &&
          m_header.dataLength - chunkBytesRead > 4)
   {
@@ -1358,13 +1406,14 @@ void libvisio::VSDXParser::readPolylineTo(WPXInputStream *input)
     return;
 
   // Default to local co-ordinates if unspecified
-  unsigned char xType = 1;
-  unsigned char yType = 1;
   std::vector<std::pair<double, double> > points;
 
   // Only formula if block is found
   if (cellRef == 2)
   {
+    unsigned char xType = 1;
+    unsigned char yType = 1;
+    unsigned long blockBytesRead = 0;
     inputPos = input->tell();
     blockBytesRead += 6;
 
@@ -1540,39 +1589,6 @@ void libvisio::VSDXParser::readSplineKnot(WPXInputStream *input)
     m_geomList->addSplineKnot(m_header.id, m_header.level, x, y, knot);
 }
 
-void libvisio::VSDXParser::readParaIX(WPXInputStream *input)
-{
-  unsigned charCount = readU32(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double indFirst = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double indLeft = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double indRight = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double spLine = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double spBefore = readDouble(input);
-  input->seek(1, WPX_SEEK_CUR);
-  double spAfter = readDouble(input);
-  unsigned char align = readU8(input);
-
-  if (m_isInStyles)
-    m_collector->collectParaIXStyle(m_header.id, m_header.level, charCount, indFirst, indLeft, indRight,
-                                    spLine, spBefore, spAfter, align);
-  else if (m_isStencilStarted)
-  {
-    VSD_DEBUG_MSG(("Found stencil paragraph style\n"));
-    if (!m_stencilShape.m_paraStyle)
-      m_stencilShape.m_paraStyle= new VSDXParaStyle(charCount, indFirst, indLeft, indRight,
-          spLine, spBefore, spAfter, align);
-  }
-  else
-    m_paraList->addParaIX(m_header.id, m_header.level, charCount, indFirst, indLeft, indRight,
-                          spLine, spBefore, spAfter, align);
-}
-
-
 void libvisio::VSDXParser::readNameList(WPXInputStream * /* input */)
 {
   if (m_isStencilStarted)
